#ifndef DDT_COMPARE_DAQHIT_H
#define DDT_COMPARE_DAQHIT_H


#include "DDTBaseDataProducts/DAQHit.h"


// This file contains the various structs that can be used to compare
// two DAQHit objects.


namespace novaddt
{
  //
  // General Template
  //
  template<class T> struct CompareDAQHit;

  //
  // Specializations of the General Template
  //
  // 1. Compare DAQ Hits based on their ADC value:
  template<> struct CompareDAQHit<ADC>
  { 
    bool operator() (DAQHit const & lhs, DAQHit const & rhs) const
    { return lhs.ADC() < rhs.ADC(); } 
    
    bool operator() (DAQHit const & lhs, ADC const & rhs) const 
    { return lhs.ADC() < rhs; }
  }; 


  // 2. Compare DAQ Hits based on their TDC value:
  template<> struct CompareDAQHit<TDC>
  { 
    bool operator() (DAQHit const & lhs, DAQHit const & rhs) const 
    {
      if (!(lhs.TDC() ==rhs.TDC())) {
	return lhs.TDC() < rhs.TDC(); 
      }
      else if (!(lhs.Plane() == rhs.Plane()) ){
	return lhs.Plane() < rhs.Plane();
      }
      else {
	return lhs.Cell() < rhs.Cell();
      }
    }
    
    bool operator() (DAQHit const & lhs, TDC const & rhs) const 
    { 
      return lhs.TDC() < rhs; 
    }
  }; 


  // 3. Compare DAQ Hits based on their Plane:
  template<> struct CompareDAQHit<Plane>
  {
    bool operator() (DAQHit const & lhs, DAQHit const & rhs) const
    { return lhs.Plane() < rhs.Plane(); }

    bool operator() (DAQHit const & lhs, Plane const & rhs) const
    { return lhs.Plane() < rhs; }
  };


  // 4. Compare DAQ Hits based on their Cell:
  template<> struct CompareDAQHit<Cell>
  {
    bool operator() (DAQHit const & lhs, DAQHit const & rhs) const
    { return lhs.Cell() < rhs.Cell(); }

    bool operator() (DAQHit const & lhs, Cell const & rhs) const
    { return lhs.Cell() < rhs; }
  };


  // 5. Compare DAQ Hits based on their View
  template<> struct CompareDAQHit<View>
  {
    bool operator() (DAQHit const & lhs, DAQHit const & rhs) const
    { return lhs.View() < rhs.View(); }

    bool operator() (DAQHit const & lhs, View const & rhs) const
    { return lhs.View() < rhs; }

    bool operator() (const DAQHit& lhs) const
    { return lhs.View().val==daqchannelmap::X_VIEW; }
  };


  // 6. Compare DAQ Hits based on their DCM
  template<> struct CompareDAQHit<DCM>
  {
    bool operator() (DAQHit const & lhs, DAQHit const & rhs) const
    { return lhs.getDCM() < rhs.getDCM(); }
    
    bool operator() (DAQHit const & lhs, DCM const & rhs) const
    { return lhs.getDCM() < rhs; }
  };
}


#endif
