#ifndef DDT_SMT_TRACK_GUARD_HH
#define DDT_SMT_TRACK_GUARD_HH

#include "DDTBaseDataProducts/DAQHit.h"

#include <map>


namespace novaddt
{
  namespace smt
  {
    class Track;
  }
}


class novaddt::smt::Track
{
public:
  Track(novaddt::DAQHit const& start, novaddt::DAQHit const& end);

  double time() const;
  double distance() const;
  double velocity() const;
  double beta() const;
  
  bool slope_plane_is_valid() const;
  double slope_cell_plane() const;
  double slope_time_plane() const;

  bool slope_time_is_valid() const;
  double slope_cell_time() const;
  double slope_plane_time() const;

  bool slope_cell_is_valid() const;
  double slope_time_cell() const;
  
  novaddt::DAQHit start() const;
  novaddt::DAQHit end() const;

  unsigned max_plane() const;
  unsigned min_plane() const;
  int plane_difference() const;
  unsigned max_cell() const;
  unsigned min_cell() const;
  int cell_difference() const;
  
private:
  double calculate_distance() const;
  double calculate_velocity() const;
  void calculate_slopes();
  
  double dp_, dc_, dt_, distance_, velocity_, beta_,
    slope_cell_plane_, slope_time_plane_,
    slope_cell_time_,  slope_plane_time_,
    slope_time_cell_;

  bool slope_plane_is_valid_, slope_time_is_valid_, slope_cell_is_valid_;
  
  novaddt::DAQHit start_, end_;
};


#endif
