#ifndef DATA_COMPRESSION_H
#define DATA_COMPRESSION_H

// This class contains static member functions which all
// for compression and decompression of Pulse Height Data

#include <math.h>

class DataCompression {

public:
  // This class can not be instantiated
  DataCompression(){
    // Populate the lookup table
    for(int i=0; i<256; ++i){
      decodeMatrix[i] = decodeVal8Bit(i);
    }
  };

  ~DataCompression(){};

  // The lookup table should be used if sequential
  // or near sequential (or the same) values are being decoded 
  // If the values are random then you invalidate the cache line
  // and it is quicker to recompute each time
  float lookup(int val){return decodeMatrix[val];}

private:

  // Utility Function
  static inline unsigned int nbitMask(unsigned int n){
    return ((~((unsigned int)0x0)
	     << (sizeof(unsigned int)*8 - n ))  
	     >> (sizeof(unsigned int)*8 - n ));
  }

  template<class t> 
  t nbitMask(unsigned int n){
    return ((~((t)0x0)
	     << (sizeof(t)*8 - n))
	    >> (sizeof(t)*8 -n));
  }

private:
  // Member Data
  float decodeMatrix[256];

public:

  static float decodeVal(unsigned input, unsigned inputBitsMantisa, 
			 int inputBitsExp)
  {
    // Check for a pass through for the low values (where we don't compress)
    if(input < unsigned(0x1 << (inputBitsMantisa + 1)) )
      return input * 1.0;

    // First get the exponent
    unsigned int expo = input >> inputBitsMantisa;
    
    // Next get the Mantisa
    unsigned int mantisa = input & nbitMask(inputBitsMantisa);
    // The value that we return includes an averaging so that
    // we minimize the error (but this converts it to a floating point)
    return (pow(2, (expo-1))*(mantisa + pow(2,inputBitsMantisa) +0.5) - 0.5);

    //return (0x1 << ((input >> inputBitsMantisa) -1)) 
    //  *((input & nbitMask(inputBitsMantisa)) + (0x1 << inputBitsMantisa) + 0.5) - 0.5;
  };

// This version assumes a 8 bit 5+3 represent
static float decodeVal8Bit(unsigned int input){

  // First get the exponent
  // unsigned int expo = input >> 5;
  
  // Next get the Mantisa
  // unsigned int mantisa = input & 0x1f;
  
  // The value that we return includes an averaging so that
  // we minimize the error (but this converts it to a floating point)
  // return (pow(2, (expo-1))*(mantisa + 32.5) - 0.5);
  
  // This line is the fully optimized version of this

  // Pass through values below our floor
  //if(input < 64){return input;}

  // This computes the value if its over the pass through floor
  // return (0x1 << ((input >> 5) -1)) *((input & 0x1f) + 32.5) - 0.5);

  // Final hand optimized version
  return (input < 64) ? input : (0x1 << ((input >> 5) -1)) *((input & 0x1f) + 32.5) - 0.5;
  };

};

#endif  /* DATA_COMPRESSION_H */
