#ifndef DDTCORE_NOVADDTRAWINPUTDRIVER_H
#define DDTCORE_NOVADDTRAWINPUTDRIVER_H

////////////////////////////////////////////////////////////////////////
/// \file  NovaDDTRawInputDriver.h
/// \brief Source to convert raw binary files to DDT root files adapted from daq2rawdigit::NOvARawInputDriver.h
///
/// \author  janzirn@fnal.gov
////////////////////////////////////////////////////////////////////////

#include "RawFileParser/RawFileParser.h"

#include "art/Persistency/Provenance/SubRunID.h"
#include "art/Framework/IO/Sources/SourceHelper.h"

#include <string>

// forward declarations
namespace novaddt {
  class NovaDDTRawInputDriver;
}

namespace fhicl{
    class ParameterSet;
}

namespace art{
    class ProductRegistryHelper;
    class RunPrincipal;
    class SubRunPrincipal;
    class EventPrincipal;
    class FileBlock;
}


class novaddt::NovaDDTRawInputDriver{
  /// Class to fill the constraints on a template argument to the class,
  /// art::Source
public:
  // Required constructor
  NovaDDTRawInputDriver(fhicl::ParameterSet const &pset,
                     art::ProductRegistryHelper &help,
                     art::SourceHelper const &pm);

  // Required by art::Source:
  void closeCurrentFile();
  void readFile(std::string const &name,
                art::FileBlock* &fb);
  bool readNext(art::RunPrincipal* const &inR,
                art::SubRunPrincipal* const &inSR,
                art::RunPrincipal* &outR,
                art::SubRunPrincipal* &outSR,
                art::EventPrincipal* &outE);

private:
  art::SourceHelper            fSourceHelper;             ///< Class to help with {Run, SubRun, Event}Principal construction.
  std::string                  fCurrentFilename;          ///< Hang on to the current filename.
  art::SubRunID                fCurrentSubRunID;          ///< Keep track of the current subRun details.
  rawfileparser::RawFileParser fParser;                   ///< Online raw file parsing utility.
  size_t                       fMaxEventSize;             ///< Will not try to read an event bigger than this.
  bool                         fFilterCorruptedNanoslices;///< Filter the corrupted nanaslices in the unpacking?
};  // NovaDDTRawInputDriver

#endif /* DDTCORE_NOVADDTRAWINPUTDRIVER_H */

