////////////////////////////////////////////////////////////////////////
// Class:       DDTDemo
// Module Type: filter
// File:        DDTDemo_module.cc
//
// an extra line
// Generated at Wed Sep 12 16:15:01 2012 by Nova DDT using artmod
// from art v1_01_01.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Persistency/Common/Assns.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Event.h"

#include "DDTBaseDataProducts/BaseProducts.h"
#include "DDTBaseDataProducts/Boundary.h"
#include "DDTBaseDataProducts/BoundaryList.h"
#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/DAQHit.h"
#include "DDTBaseDataProducts/GroupedHitList.h"
#include "DDTBaseDataProducts/TriggerDecision.h"

#include <string>
#include <algorithm>

namespace novaddt {
  class DDTDemo;
}

class novaddt::DDTDemo : public art::EDFilter {
public:
  explicit DDTDemo(fhicl::ParameterSet const & p);
  virtual ~DDTDemo();

  virtual bool filter(art::Event & e);

private:
  std::string _slicelabel;
  std::string _sliceinstance;
  unsigned _prescale;
  unsigned _trigger_counts;
};


novaddt::DDTDemo::DDTDemo(fhicl::ParameterSet const & p)
  : _slicelabel(p.get<std::string>("slice_label")),
    _sliceinstance(p.get<std::string>("slice_instance")),
    _prescale(p.get<unsigned>("prescale", 1)),
    _trigger_counts(0)
{
  produces<std::vector<TriggerDecision>>();
}

novaddt::DDTDemo::~DDTDemo()
{
  // Clean up dynamic memory and other resources here.
}

bool novaddt::DDTDemo::filter(art::Event & e)
{
  bool result = false;

  std::unique_ptr<std::vector<TriggerDecision>> 
    trigger_decisions(new std::vector<TriggerDecision>);

  art::Handle<GroupedHitList<TDC>> slices;
  e.getByLabel(_slicelabel, _sliceinstance, slices);

  unsigned n_slices = slices->n_groups();

  for (unsigned slice = 0; slice != n_slices; ++slice)
  {
    if (slices->group_size(slice) > 1) // > 42)
    {
      HitList hits = slices->get(slice);
      TDC t_begin = hits.front().TDC();
      TDC t_end = hits.back().TDC();

      ++_trigger_counts;
      if (_trigger_counts % _prescale == _prescale - 1)
	trigger_decisions->
	  emplace_back(t_begin, t_end - t_begin,
		       daqdataformats::trigID::TRIG_ID_DATA_SN, _prescale);

      // This event contains at least one useful slice, so let's keep it around:
      result = true;
    }
  }

  e.put(std::move(trigger_decisions));

  return result;
}

DEFINE_ART_MODULE(novaddt::DDTDemo)
