////////////////////////////////////////////////////////////////////////
// Class:       DDTStreamer
// Module Type: filter
// File:        DDTStreamer_module.cc
//
// an extra line
// Generated at Wed Sep 12 16:15:01 2012 by Nova DDT using artmod
// from art v1_01_01.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Persistency/Common/Assns.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Event.h"

#include "DDTBaseDataProducts/BaseProducts.h"
#include "DDTBaseDataProducts/Boundary.h"
#include "DDTBaseDataProducts/BoundaryList.h"
#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/DAQHit.h"
#include "DDTBaseDataProducts/TriggerDecision.h"

#include <string>
#include <algorithm>

//---------------------------------------------------------------------
namespace novaddt {
    class DDTStreamer;
}
//---------------------------------------------------------------------
class novaddt::DDTStreamer : public art::EDFilter {
    public:
        explicit DDTStreamer(fhicl::ParameterSet const & p);
        virtual ~DDTStreamer();        
        virtual bool filter(art::Event & e);
        void    endJob() override;
    private:
        unsigned int _prescale;           ///< prescale factor (1 out of every this many passes are issued as real triggers)
        std::string _slicelabel;
        std::string _sliceinstance;
        unsigned int _nEvents           = 0;
        unsigned int _trigger_counts    = 0;
        unsigned int _prescaled_trigger_counts = 0;
};
//---------------------------------------------------------------------

novaddt::DDTStreamer::DDTStreamer(fhicl::ParameterSet const & p)
  :   _prescale          (p.get<unsigned int>("prescale")),
      _slicelabel(p.get<std::string>("slice_label")),
      _sliceinstance(p.get<std::string>("slice_instance"))
{
    std::cout << "--- novaddt::DDTStreamer instantiate" << std::endl;
    std::cout << "\t prescale:                  " << _prescale       << std::endl;    
    std::cout << "\t SliceModuleLabel:          " << _slicelabel     << std::endl;
    std::cout << "\t SliceInstanceLabel:        " << _sliceinstance  << std::endl;
    produces<std::vector<TriggerDecision>>();
}
//---------------------------------------------------------------------
novaddt::DDTStreamer::~DDTStreamer()
{
    // Clean up dynamic memory and other resources here.
}
//---------------------------------------------------------------------
bool novaddt::DDTStreamer::filter(art::Event & e)
{
    //std::cout << "    DDTStreamer::filter. Event: " 
    //          << e.id().event() 
    //          << std::endl;
    _nEvents++;
    bool result = false;
    
    // static unsigned event = 1;
    // event++;
    // if (event == 13)
    // {
    //   std::cout << "\nSleeping now!\n" << std::endl;
    //   sleep(60);
    // }
    
    std::unique_ptr<std::vector<TriggerDecision>> 
    trigger_decisions(new std::vector<TriggerDecision>);
    
    art::Handle<novaddt::HitList> hits;
    e.getByLabel(_slicelabel, _sliceinstance, hits);
    
    //std::cout << "\tgot " << hits->size() << " hits" << std::endl;
    // This event contains at least one useful slice, so let's keep it around:
    result = true;
    _trigger_counts++;
    
    // check the prescale
    if (_trigger_counts%_prescale == _prescale-1) {
        _prescaled_trigger_counts++;
        //std::cout<< "\t\t - not prescaled" << std::endl;
    
        TDC t_begin = hits->front().TDC();
        TDC t_end = hits->back().TDC();
        
        std::cout << "\tbegin: " << t_begin.val 
                << ", end: "     << t_end.val
                << std::endl;
        
	/*
	  This streamer trigger will NOT WORK because of the GT design.
	  In DDT, we send all of our trigger messages to the DDTServerMailbox,
	  but the manual bit, which is being sent below, needs to be sent
	  to the ManServerMailbox.

	  Alec suggests that we use the ManSender instead to collect streamer
	  data.
	 */

        // trigger_decisions->emplace_back(t_begin, t_end - t_begin,
        //                                 daqdataformats::trigID::TRIG_ID_SOFT_MANUAL,
        //                                 _prescale);
    } // end of check on prescale
    
    e.put(std::move(trigger_decisions));
    
    return result;
}
//---------------------------------------------------------------------
void novaddt::DDTStreamer::endJob()
{
    std::cout << "--- novaddt::DDTStreamer endJob" << std::endl;
    std::cout << "\tNumber of events seen:        " << _nEvents            << std::endl;
    std::cout << "\tNumber that pass the trigger: " << _trigger_counts      << std::endl;
    std::cout << "\t              after prescale: " << _prescaled_trigger_counts << std::endl;
}
//---------------------------------------------------------------------
DEFINE_ART_MODULE(novaddt::DDTStreamer)
