////////////////////////////////////////////////////////////////////////
// Class:       Evaluation
// Module Type: analyzer
// File:        Evaluation_module.cc
//
// Generated at Mon Mar  4 13:36:31 2013 by Martin Frank using artmod
// from art v1_02_06.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Event.h"

#include "DDTBaseDataProducts/BaseProducts.h"
#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/GroupedHitList.h"

#include <algorithm>
#include <fstream>

namespace novaddt {
  class Evaluation;
  struct Hit;
  struct CompareHit;
}

class novaddt::Evaluation : public art::EDAnalyzer {
public:
  explicit Evaluation(fhicl::ParameterSet const & p);
  virtual ~Evaluation();

  void analyze(art::Event const & e) override;


private:
  std::string _slicelabel;
  std::string _sliceinstance;
};

struct novaddt::Hit 
{
  Hit(unsigned short p, unsigned short c, int32_t t, int16_t a) :
    plane(p), cell(c), tdc(t), adc(a) {}

  unsigned short plane, cell;
  int32_t tdc;
  int16_t adc;
};

struct novaddt::CompareHit
{
    // sort in this order: 1. plane  2. cell  3. tdc
  bool operator() (Hit const& lhs, Hit const& rhs)
  {
    if (lhs.plane == rhs.plane)
    {
      if (lhs.cell == rhs.cell)
	return lhs.tdc < rhs.tdc;
      
      return lhs.cell < rhs.cell;
    }
 
    return lhs.plane < rhs.plane;
  }
};

novaddt::Evaluation::Evaluation(fhicl::ParameterSet const & p) :   
  EDAnalyzer(p),
  _slicelabel(p.get<std::string>("slice_label")),
  _sliceinstance(p.get<std::string>("slice_instance"))
{
}

novaddt::Evaluation::~Evaluation()
{
}

void novaddt::Evaluation::analyze(art::Event const & e)
{
  art::Handle<GroupedHitList<TDC>> slices;
  e.getByLabel(_slicelabel, _sliceinstance, slices);

  std::vector<Hit> ohits;

  unsigned n_slices = slices->n_groups();
  for (unsigned slice = 0; slice != n_slices; ++slice)
  {
    HitList hits = slices->get(slice);
    // TDC t0 = hits.at(0).TDC();

    for (auto const& hit : hits)
    {
      std::cout << "\nPlane = " << hit.Plane().val
		<< "\nView  = " << hit.View().val
		<< std::endl;
      ohits.emplace_back(hit.Plane().val, hit.Cell().val, 
			 hit.TDC().val, hit.ADC().val);
    }
  }

  static unsigned n_event = 1;
  HitList first_hits = slices->get(0);
  HitList last_hits = slices->get(n_slices - 1);
  std::cout << "\nEvent " << n_event++
	    << "\nFirst:"
	    << "\n\tt_begin = " << first_hits.front().TDC().val
	    << "\nLast:"
	    << "\n\tt_end = " << last_hits.back().TDC().val
	    << std::endl;

  sort(ohits.begin(), ohits.end(), CompareHit());
  
  std::ofstream outfile("ddt.txt");
  std::string spacer = " ";
  for (auto const& ohit : ohits)
    outfile << ohit.plane << spacer
	    << ohit.cell << spacer
	    << ohit.tdc << spacer
	    << ohit.adc << std::endl;

  outfile.close();
}

DEFINE_ART_MODULE(novaddt::Evaluation)
