////////////////////////////////////////////////////////////////////////
// Class:       EventPrescale
// Module Type: filter
// File:        EventPrescale_module.cc
//
// Generated at Thu Feb 13 05:14:01 2014 by Matthew Tamsett using artmod
// from cetpkgsupport v1_04_02.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"

#include <iostream>
#include <memory>

#include <stdlib.h> // for rand
#include <time.h>   // for rand seed
//---------------------------------------------------------------------
namespace novaddt {
    class EventPrescale;
}
//---------------------------------------------------------------------
class novaddt::EventPrescale : public art::EDFilter {
    public:
        explicit EventPrescale(fhicl::ParameterSet const & p);
        virtual ~EventPrescale();
        bool filter(art::Event & e) override;
        void endJob() override;
    private:
        // Declare member data here.
        // Configuration options
        unsigned int _prescale;           ///< prescale factor (1 out of every this many passes are issued as real triggers)
        std::string  _mode;               ///< prescale mode, can be either fixed or random
        unsigned int _seed;               ///< random number generator seed
        unsigned int _nEvents           = 0;
        unsigned int _nAccepted         = 0;
        
};
//---------------------------------------------------------------------
novaddt::EventPrescale::EventPrescale(fhicl::ParameterSet const & p)
:   _prescale          (p.get<unsigned int>("prescale"              ))
,   _mode              (p.get<std::string >("mode"                  ))
,   _seed              (p.get<unsigned int>("seed",       time(NULL)))
{
    // Call appropriate Produces<>() functions here.
    std::cout << "--- novaddt::EventPrescale instantiate" << std::endl;
    std::cout << "\t prescale:                      " << _prescale << std::endl;
    std::cout << "\t mode:                          " << _mode     << std::endl;
    std::cout << "\t seed:                          " << _seed     << std::endl;
    
    // assert that the mode requested is configured
    assert( (_mode == "fixed") || (_mode == "random") );
    
    // seed the random number generator, use a random default if not specified    
    // in the configuration file.  
    srand(_seed);
}
//---------------------------------------------------------------------
novaddt::EventPrescale::~EventPrescale()
{
    // Clean up dynamic memory and other resources here.
}
//---------------------------------------------------------------------
bool novaddt::EventPrescale::filter(art::Event & event)
{
    LOG_DEBUG("EventPrescale")<< "--- novaddt::EventPrescale filter. Event: " 
              << event.id().event() 
              << std::endl;
    _nEvents++;
    
    // check prescale
    if ( ((_mode=="fixed")  && ( _nEvents%_prescale == _prescale-1)) ||
         ((_mode=="random") && (rand()%_prescale    == _prescale-1)) ){
            _nAccepted++;
            LOG_DEBUG("EventPrescale")<< "\t - not prescaled" << std::endl;
            return true;
    } // end of check on prescale
    
    // if not accepted by prescale return false
    return false;
}
//---------------------------------------------------------------------
void novaddt::EventPrescale::endJob()
{
    std::cout << "--- novaddt::EventPrescale endJob" << std::endl;
    std::cout << "\tNumber of events seen:         " << _nEvents             << std::endl;
    std::cout << "\tNumber of events accepted:     " << _nAccepted           << std::endl;
}
//---------------------------------------------------------------------
DEFINE_ART_MODULE(novaddt::EventPrescale)
