////////////////////////////////////////////////////////////////////////
// Class:       FastMMTrigger
// Module Type: filter
// File:        FastMMTrigger_module.cc
//
// Generated at Mon Dec 17 15:29:53 2012 by Zukai Wang using artmod
// from art v1_02_06.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Persistency/Common/Assns.h"
#include "art/Framework/Principal/Handle.h"	
#include "art/Framework/Principal/Event.h"
#include "DDTUtilities/AssociationUtil.h"

#include "DDTBaseDataProducts/BaseProducts.h"
#include "DDTBaseDataProducts/Boundary.h"
#include "DDTBaseDataProducts/BoundaryList.h"
#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/DAQHit.h"
#include "DDTBaseDataProducts/GroupedHitList.h"
#include "DDTBaseDataProducts/TriggerDecision.h"
#include "DDTBaseDataProducts/Track.h"	

#include <string>
#include <vector>
#include <algorithm>

namespace novaddt {
  class FastMMTrigger;
}

class novaddt::FastMMTrigger : public art::EDFilter {
public:
  explicit FastMMTrigger(fhicl::ParameterSet const & p);
  virtual ~FastMMTrigger();

  virtual bool filter(art::Event & e) override;

private:
  unsigned _prescale;
  std::string _slicelabel;
  std::string _sliceinstance;
  int _ADCthr;
  int _meanADCthr;
  int _xMin;  
  int _xMax;
  int _yMin;
  int _yMax;
  int _zMin;
  int _zMax;
  int _deltx;
  int _delty;
  int _deltz;
  int _Nsh;
  int _sigmaP;
  int _TBuffer;

  int PX_max, PY_max, PX_min, PY_min;
  TDC::value_type TX_max, TY_max, TX_min, TY_min;

  unsigned _trigger_counts; 
};

novaddt::FastMMTrigger::FastMMTrigger(fhicl::ParameterSet const & p)
  :
  _prescale(p.get<unsigned>("prescale")),
  _slicelabel(p.get<std::string>("slice_label")),
  _sliceinstance(p.get<std::string>("slice_instance")),
  _ADCthr(p.get<int>("ADC_thr")),
  _meanADCthr(p.get<int>("mean_ADCthr")),
  _xMin(p.get<int>("x_min")),
  _xMax(p.get<int>("x_max")),
  _yMin(p.get<int>("y_min")),
  _yMax(p.get<int>("y_max")),
  _zMin(p.get<int>("z_min")),
  _zMax(p.get<int>("z_max")),
  _deltx(p.get<int>("x_delt")),
  _delty(p.get<int>("y_delt")),
  _deltz(p.get<int>("z_delt")),
  _Nsh(p.get<int>("N_sh")),
  _sigmaP(p.get<int>("sigma_P")),
  _TBuffer(p.get<TDC::value_type>("T_Buffer")),
  _trigger_counts(0)
{
  // Call appropriate Produces<>() functions here.
  produces<std::vector<TriggerDecision>>();
}

novaddt::FastMMTrigger::~FastMMTrigger()
{
  // Clean up dynamic memory and other resources here.
}

bool novaddt::FastMMTrigger::filter(art::Event & e)	
{
  bool result = false;
  std::unique_ptr<std::vector<TriggerDecision>> 
    trigger_decisions(new std::vector<TriggerDecision>);

  art::Handle<std::vector<novaddt::HitList> > slices;
  e.getByLabel(_slicelabel, _sliceinstance, slices);
//  std::cout<<"number of slices you are getting: "<<slices->size()<<std::endl;
  for (unsigned i = 0; i!= slices->size(); ++i) {
    HitList const &hits = (*slices)[i];
    PX_max = 0;
    PY_max = 0;
    PX_min = 99999;
    PY_min = 99999;
    TX_max = hits.front().TDC().val;
    TY_max = TX_max;
    TX_min = hits.back().TDC().val;
    TY_min = TX_min;

    int ADC_sum = 0;
    int nb_hits = hits.size();
    int nb_high = 0;
    //  std::cout<<"nb of hits: "<<nb_hits<<std::endl;
    int  entry_id = 0;
    bool PassThrough = false;
    bool Intrusion   = false;

    int nx = 0;
    int ny = 0;

    for (auto const hit: hits) {
      if (hit.ADC().val > _ADCthr) {
	/*
	std::cout<<"fmm ADC: "<<hit.ADC().val
		 <<"   Plane: "<<hit.Plane().val
		 <<"   Cell:  "<<hit.Cell().val<<std::endl;
	*/
	++nb_high;
      }      
      ADC_sum += hit.ADC().val; 
 
      //X view hits
      if (hit.View().val == daqchannelmap::X_VIEW) {
	//	std::cout<<"X hits"<<std::endl;
	++nx;
	if (PX_max < hit.Plane().val)   PX_max  = hit.Plane().val;
	if (PX_min > hit.Plane().val)   PX_min  = hit.Plane().val;
	if (TX_max < hit.TDC().val)     TX_max  = hit.TDC().val;
	if (TX_min > hit.TDC().val)     TX_min  = hit.TDC().val;  
	
	if (!Intrusion) {
	  if (hit.Cell().val < _xMin+_deltx) {
	    entry_id  = 1;
	    Intrusion = true;
	  }
	  else if (hit.Cell().val> _xMax-_deltx) {
	    entry_id  = 2;
	    Intrusion = true;
	  }
	}
	else if (!PassThrough) {
	  if (hit.Cell().val < _xMin+_deltx && entry_id != 1) PassThrough = true;
	  else if (hit.Cell().val> _xMax-_deltx && entry_id != 2) PassThrough = true;
	}
      } 

      //Y view hits
      else {
	//	std::cout<<"Y hits"<<std::endl;
	++ny;
	if (PY_max < hit.Plane().val)   PY_max   = hit.Plane().val;
	if (PY_min > hit.Plane().val)   PY_min   = hit.Plane().val;
	if (TY_max < hit.TDC().val )   TY_max   = hit.TDC().val;
	if (TY_min > hit.TDC().val )   TY_min   = hit.TDC().val;

	if (!Intrusion) {
	  if (hit.Cell().val < _yMin+_delty) {
	    entry_id  = 3;
	    Intrusion = true;
	  }
	  else if (hit.Cell().val> _yMax-_delty) {
	    entry_id  = 4;
	    Intrusion = true;
	  }
	}
	else if (!PassThrough) {
	  if (hit.Cell().val < _yMin+_delty && entry_id != 3) PassThrough = true;
	  else if (hit.Cell().val> _yMax-_delty && entry_id != 4) PassThrough = true;
	}
      }

      //No matter which view:
      if (!Intrusion) {
	if (hit.Plane().val < _zMin+_deltz) {
	    entry_id  = 5;
	    Intrusion = true;
	}
	else if (hit.Plane().val > _zMax-_deltz) {
	    entry_id  = 6;
	    Intrusion = true;
	}
      }

      else if (!PassThrough) {
	if (hit.Plane().val < _zMin+_deltz && entry_id != 5) PassThrough=true;
	else if (hit.Plane().val > _zMax-_deltz && entry_id != 6) PassThrough=true;
      }
    }


    //Min-hits on both view:
    //   std::cout<<"xhits: "<<nx<<"   yhits: "<<ny<<std::endl;

    if (nx<3 || ny<3) continue;
    //    std::cout<<"minimum hits requirement met!"<<std::endl;

    //Passing Through Test:
    if (!PassThrough) continue;
    //    std::cout<<"through!"<<std::endl;

    //Matching Test:
    if (std::min(PX_max,PY_max) < std::max(PX_min,PY_min)+_sigmaP) continue; //Require minimum overlapping planes
    if (std::min(TX_max,TY_max) < std::max(TX_min,TY_min) ) continue;

    if (ADC_sum > nb_hits*_meanADCthr && nb_high>_Nsh) {
    //      std::cout<<"Number of saturated hits: "<<nb_high<<"   mean ADC: "<<ADC_sum/(nb_hits+0.)<<std::endl;
      unsigned dT = std::max(TX_max,TY_max)-std::min(TX_min,TY_min);

     // std::cout<<"dT: "<<dT<<" TDC"<<std::endl;
      ++_trigger_counts;
      if (_trigger_counts%_prescale == _prescale-1) {

	trigger_decisions->
	  emplace_back(std::min(TX_min,TY_min), 
		       dT,  
		       daqdataformats::trigID::TRIG_ID_DATA_FASTMONO, _prescale);
	result = true;
      }
    }
  }

  e.put(std::move(trigger_decisions));
  return result;
}



DEFINE_ART_MODULE(novaddt::FastMMTrigger)
