////////////////////////////////////////////////////////////////////////
// Class:       HighETrigger
// Module Type: filter
// File:        HighETrigger_module.cc
//
// Generated at Mon Dec 17 15:29:53 2012 by Zukai Wang using artmod
// from art v1_02_06.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Persistency/Common/Assns.h"
#include "art/Framework/Principal/Handle.h"	
#include "art/Framework/Principal/Event.h"
#include "DDTUtilities/AssociationUtil.h"

#include "DDTBaseDataProducts/CompareDAQHit.h"
#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/TriggerDecision.h"

#include <string>
#include <vector>
#include <algorithm>

namespace novaddt {
  class HighETrigger;
}

class novaddt::HighETrigger : public art::EDFilter {
public:
  explicit HighETrigger(fhicl::ParameterSet const & p);
  virtual ~HighETrigger();

  virtual bool filter(art::Event & e) override;

private:
  bool send_trigger(unsigned const sum_adc, 
		    unsigned const n_hits_total) const;

  std::string slice_label_, slice_instance_;
  unsigned prescale_, trigger_counts_;
  unsigned sum_adc_trigger_threshold_;
  unsigned n_hits_trigger_threshold_;
};

novaddt::HighETrigger::HighETrigger(fhicl::ParameterSet const & p) :
  slice_label_(p.get<std::string>("slice_label")),
  slice_instance_(p.get<std::string>("slice_instance")),
  prescale_(p.get<unsigned>("prescale")),
  trigger_counts_(0),
  sum_adc_trigger_threshold_
  (p.get<unsigned>("sum_adc_trigger_threshold")),
  n_hits_trigger_threshold_
  (p.get<unsigned>("n_hits_trigger_threshold"))
{
  produces<std::vector<TriggerDecision>>();
}

novaddt::HighETrigger::~HighETrigger()
{
  std::cout << "\n\tNumber of High Energy Triggers = " << trigger_counts_
  	    << "\n" << std::endl;
}

bool novaddt::HighETrigger::filter(art::Event & e)	
{
  std::unique_ptr<std::vector<TriggerDecision>> 
    trigger_decisions(new std::vector<TriggerDecision>);

  art::Handle<std::vector<novaddt::HitList> > slices;
  e.getByLabel(slice_label_, slice_instance_, slices);

  std::vector<novaddt::HitList> triggered_slices;

  for (auto const& slice : *slices)
  {
    unsigned sum_adc = 0;

    for (auto const& hit : slice)
      sum_adc += hit.ADC().val;

    double n_hits_total = slice.size();
    
    if (send_trigger(sum_adc, n_hits_total))
      triggered_slices.push_back(slice);
  }

  if (triggered_slices.size() == 0)
    return false;

  // initialize the absolute min and max to a reasonable TDC value,
  // so let's just use the first hit:
  novaddt::TDC absolute_t0 = triggered_slices.front().front().TDC();
  novaddt::TDC absolute_t1 = absolute_t0;
  for (auto const& t_slice : triggered_slices)
  {
    auto time_interval = 
      std::minmax_element(t_slice.begin(), t_slice.end(), CompareDAQHit<TDC>());
    auto t0 = time_interval.first->TDC();
    auto t1 = time_interval.second->TDC();

    if (t0 < absolute_t0)
      absolute_t0 = t0;

    if (t1 > absolute_t1)
      absolute_t1 = t1;
  }      
      
  ++trigger_counts_;
  if (trigger_counts_ % prescale_ == prescale_ - 1) 
  {
    auto dt = absolute_t1 - absolute_t0;
	
    trigger_decisions->emplace_back
      (absolute_t0, dt, daqdataformats::trigID::TRIG_ID_DATA_ENERGY, prescale_);
  }

  e.put(std::move(trigger_decisions));
  return true;
}

bool novaddt::HighETrigger::send_trigger
(unsigned const sum_adc,
 unsigned const n_hits_total) const
{
  if (sum_adc < sum_adc_trigger_threshold_)
    return false;

  if (n_hits_total < n_hits_trigger_threshold_)
    return false;

  return true;
}

DEFINE_ART_MODULE(novaddt::HighETrigger)
