////////////////////////////////////////////////////////////////////////
// Class:       TimeSlice
// Module Type: producer
// File:        TimeSlice_module.cc
//
// Generated at Fri Oct  5 15:21:42 2012 by Evan Niner using artmod
// from art v1_01_01.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Persistency/Common/Assns.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Event.h"

#include "DDTBaseDataProducts/BaseProducts.h"
#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/DAQHit.h"
#include "DDTBaseDataProducts/GroupedHitList.h"

#include <string>
#include <algorithm>

namespace novaddt {
  class TimeSlice;
}

class novaddt::TimeSlice : public art::EDFilter{
public:
  explicit TimeSlice(fhicl::ParameterSet const & p);
  virtual ~TimeSlice();
  GroupedHitList<TDC> MakeSlice(art::Handle<novaddt::HitList> &hits);
  virtual bool filter(art::Event & e);


private:
  std::string _hitslabel;
  std::string _instance;
  double fTimeWindow;
  unsigned int fMinHits;
  // Declare member data here.

};


novaddt::TimeSlice::TimeSlice(fhicl::ParameterSet const & p)
:
_hitslabel (p.get< std::string >("hits_label")),
_instance  (p.get< std::string >("instance"  )),
fTimeWindow(p.get< double      >("TimeWindow")),
fMinHits(   p.get< int         >("MinHits"   ))
// Initialize member data here.
{
  // Call appropriate Produces<>() functions here.
  produces<GroupedHitList<TDC>>("TDCSlice");
}

novaddt::TimeSlice::~TimeSlice()
{
  // Clean up dynamic memory and other resources here.
}

bool novaddt::TimeSlice::filter(art::Event & e)
{
  art::Handle<novaddt::HitList> hits;
  e.getByLabel(_hitslabel, _instance, hits);

  //make a grouped hitlist out of hits after time slicing
  std::unique_ptr< GroupedHitList<TDC> >product(new GroupedHitList<TDC>);

  if(hits->size() > 0) *product = MakeSlice(hits);

  //put the hits into the event for later use
  e.put(std::move(product),"TDCSlice");

  return true;
}

// it is assumed that a valid HitList (ie one with entries) is passed to this method
novaddt::GroupedHitList<novaddt::TDC> novaddt::TimeSlice::MakeSlice(art::Handle<novaddt::HitList> &hits)
{
  std::vector<size_t> boundaries; //list of boundary points for timeslices

  //must have a boundary at start of list
  boundaries.push_back(0);

  //range of hits to put into slice
  auto itStart = hits->begin();
  auto itEnd   = hits->begin();
  //testpoint to expand slice
  auto itTest  = hits->begin();

  while(true){
    //grow slice till its size of fTimeWindow
    while(itEnd->TDC().val - itStart->TDC().val < fTimeWindow){
      ++itEnd;
      if(itEnd == hits->end()) break; //out of hits
    }
    if(itEnd == hits->end()) break; //break all the way out
    
    //not enough hits in window, drop first and try again;
    if(itEnd - itStart < fMinHits){
      ++itStart;
      assert(itStart <= itEnd);
      //add boundary to partition off this hit
      boundaries.push_back(std::distance(hits->begin(),itStart));
      continue;
    }

    //windown is long enough and has enough change, catch itTest up
    //until it defines the start of a fTimeWindow-sized window ending
    //on itEnd
    while(itEnd->TDC().val - itTest->TDC().val > fTimeWindow) ++itTest;

    //if growing the window would cause itTest-itEnd window to
    //not have enough hits, write out
    if(itEnd+1 - itTest < fMinHits){
      boundaries.push_back(std::distance(hits->begin(), itEnd));
      itStart = itEnd;
      continue;
    }
  
    //can still legally grow window,do that
    ++itEnd;
    if(itEnd == hits->end()) break;
  }

  //now add boundary for end of hitlist
  boundaries.push_back(std::distance(hits->begin(),hits->end()));


  return GroupedHitList<TDC>(hits->begin(), hits->end(), boundaries);
}

DEFINE_ART_MODULE(novaddt::TimeSlice)
