////////////////////////////////////////////////////////////////////////
// Class:       SupernovaTrigger
// Module Type: filter
// File:        SupernovaTrigger_module.cc
//
// Generated at Mon Dec 17 15:29:53 2012 by Zukai Wang using artmod
// from art v1_02_06.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Persistency/Common/Assns.h"
#include "art/Framework/Principal/Handle.h"	
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Services/Optional/TFileService.h" 

#include "DDTBaseDataProducts/BaseProducts.h"
#include "DDTBaseDataProducts/Boundary.h"
#include "DDTBaseDataProducts/BoundaryList.h"
#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/DAQHit.h"
#include "DDTBaseDataProducts/GroupedHitList.h"
#include "DDTBaseDataProducts/TriggerDecision.h"	
#include "DDTBaseDataProducts/EventHeader.h"	
#include "DDTUtilities/AssociationUtil.h"

#include "DAQMessages/ccpp_NSNMessages.h"
#include "DAQMessages/DDSMailbox.hpp"

#include <string>
#include <queue>
#include <algorithm>


#include "TH1.h"

namespace novaddt {
  class SupernovaTrigger;
}

class novaddt::SupernovaTrigger : public art::EDFilter {
public:
  explicit SupernovaTrigger(fhicl::ParameterSet const & p);
  virtual ~SupernovaTrigger();
  virtual bool filter(art::Event & e) override;
  virtual void endJob() override;

  uint64_t GetHits(const art::Event & e);
  uint64_t GetTime(const art::Event & e);
  bool SliceInSignalRegion(const novaddt::HitList& slice);
private:
  unsigned _adc_min;
  unsigned _adc_max;
  std::string _slice_label;
  std::string _slice_instance;
  std::string _hits_label;
  std::string _hits_instance;
  
  unsigned _msg_buffer_idx;
  nsnmessages::NSNPackedMessage _message;
  DDSOutbox<nsnmessages::NSNPackedMessage> _mailbox;
// some histograms for statistics:
  TH1D* _Hist_ADC;
//constants
  const unsigned _msg_buffer_size=10;
};

novaddt::SupernovaTrigger::SupernovaTrigger(fhicl::ParameterSet const & p):
  _adc_min (p.get<unsigned>("adc_min")),
  _adc_max (p.get<unsigned>("adc_max")),
  _slice_label    (p.get< std::string >("slice_label"     )),
  _slice_instance (p.get< std::string >("slice_instance"  )),
  _hits_label     (p.get< std::string >("hits_label"      )),
  _hits_instance  (p.get< std::string >("hits_instance"   )),
 
  _msg_buffer_idx(0),
  _mailbox("SuperNovaDDT",
           p.get<unsigned>("dds_partition"),
           p.get<std::string>("dds_mailbox"))

{
  // Call appropriate Produces<>() functions here.
  // produces<std::vector<TriggerDecision>>("TriggerDecisions");
  art::ServiceHandle<art::TFileService> tfs;
  _Hist_ADC=tfs->make<TH1D>("hADC","Slices ADC; ADC",100,0,20000);
}

novaddt::SupernovaTrigger::~SupernovaTrigger()
{
  // Clean up dynamic memory and other resources here.
}

uint64_t novaddt::SupernovaTrigger::GetHits(const art::Event & e)
{
    uint64_t hits=0;
    /*art::Handle<novaddt::EventHeader> header;
    e.getByLabel("DDTdaq",header);
    hits= header->eventNumber;*/
 
  art::Handle<std::vector<novaddt::HitList> > slices;
  e.getByLabel(_slice_label, _slice_instance, slices);
  //calculate hits in signal region
  for (auto const& slice : *slices)
     if(SliceInSignalRegion(slice))hits++;
  return hits;
}

uint64_t novaddt::SupernovaTrigger::GetTime(const art::Event & e){
    const uint64_t msec=64000; //TDC ticks/msec
    art::Handle<novaddt::EventHeader> header;
    e.getByLabel("DDTdaq",header);
    return header->timeStart/msec;
   // header.getByLabel(_hits_label, _hits_instance, hits);
   // auto t0=hits->front().TDC().val;
   // auto t1=hits->back(). TDC().val;
   // std::cout<<"time=["<<t0<<" - "<<t1<<"] = "<<t1-t0<<std::endl;
   // return t0;
}

bool novaddt::SupernovaTrigger::SliceInSignalRegion(const novaddt::HitList& slice){
   //calculate total slice ADC
   unsigned adc_sum=0;
   for(auto const& hit : slice){
      adc_sum+=hit.ADC().val;   
   }
   _Hist_ADC->Fill(adc_sum);
   //check if adc value for slice is whithin limits
   return (adc_sum>_adc_min) && (adc_sum<_adc_max);
}

bool novaddt::SupernovaTrigger::filter(art::Event & e)
{
  _message.time[_msg_buffer_idx]=GetTime(e);
  _message.hits[_msg_buffer_idx]=GetHits(e);
  _msg_buffer_idx++;

  if(_msg_buffer_idx>=_msg_buffer_size){
    //it's time to send message!
    _msg_buffer_idx=0;
    _mailbox<<_message; //send message
    }
  return false;
}


void novaddt::SupernovaTrigger::endJob()
{
    //send empty message to signalize end of the work to GlobalTrigger
    for(auto & t: _message.time) t=0;
    for(auto & h: _message.hits) h=0;
    _mailbox<<_message;
}
DEFINE_ART_MODULE(novaddt::SupernovaTrigger)
